@@ -1,18 +1,17 @@
 //Copyright (c) 2018 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
+#include "ExtruderTrain.h"
+#include "sliceDataStorage.h"
 #include "WallsComputation.h"
+#include "settings/types/Ratio.h"
 #include "utils/polygonUtils.h"
+
 namespace cura {
 
-WallsComputation::WallsComputation(int wall_0_inset, int line_width_0, int line_width_x, int insetCount, bool recompute_outline_based_on_outer_wall, bool remove_parts_with_no_insets, const bool try_line_thickness)
-: wall_0_inset(wall_0_inset)
-, line_width_0(line_width_0)
-, line_width_x(line_width_x)
-, insetCount(insetCount)
-, recompute_outline_based_on_outer_wall(recompute_outline_based_on_outer_wall)
-, remove_parts_with_no_insets(remove_parts_with_no_insets)
-, try_line_thickness(try_line_thickness)
+WallsComputation::WallsComputation(const Settings& settings, const LayerIndex layer_nr)
+: settings(settings)
+, layer_nr(layer_nr)
 {
 }
 
@@ -24,14 +23,37 @@ WallsComputation::WallsComputation(int wall_0_inset, int line_width_0, int line_
  */
 void WallsComputation::generateInsets(SliceLayerPart* part)
 {
-    if (insetCount == 0)
+    size_t inset_count = settings.get<size_t>("wall_line_count");
+    const bool spiralize = settings.get<bool>("magic_spiralize");
+    if (spiralize && layer_nr < LayerIndex(settings.get<size_t>("initial_bottom_layers")) && ((layer_nr % 2) + 2) % 2 == 1) //Add extra insets every 2 layers when spiralizing. This makes bottoms of cups watertight.
+    {
+        inset_count += 5;
+    }
+    if (settings.get<bool>("alternate_extra_perimeter"))
+    {
+        inset_count += ((layer_nr % 2) + 2) % 2;
+    }
+
+    if (inset_count == 0)
     {
         part->insets.push_back(part->outline);
         part->print_outline = part->outline;
         return;
     }
 
-    for(int i=0; i<insetCount; i++)
+    const coord_t wall_0_inset = settings.get<coord_t>("wall_0_inset");
+    coord_t line_width_0 = settings.get<coord_t>("wall_line_width_0");
+    coord_t line_width_x = settings.get<coord_t>("wall_line_width_x");
+    if (layer_nr == 0)
+    {
+        const ExtruderTrain& train_wall_0 = settings.get<ExtruderTrain&>("wall_0_extruder_nr");
+        line_width_0 *= train_wall_0.settings.get<Ratio>("initial_layer_line_width_factor");
+        const ExtruderTrain& train_wall_x = settings.get<ExtruderTrain&>("wall_x_extruder_nr");
+        line_width_x *= train_wall_x.settings.get<Ratio>("initial_layer_line_width_factor");
+    }
+
+    const bool recompute_outline_based_on_outer_wall = (settings.get<bool>("support_enable") || settings.get<bool>("support_tree_enable")) && !settings.get<bool>("fill_outline_gaps");
+    for(size_t i = 0; i < inset_count; i++)
     {
         part->insets.push_back(Polygons());
         if (i == 0)
@@ -50,7 +72,7 @@ void WallsComputation::generateInsets(SliceLayerPart* part)
         const size_t inset_part_count = part->insets[i].size();
         constexpr size_t minimum_part_saving = 3; //Only try if the part has more pieces than the previous inset and saves at least this many parts.
         constexpr coord_t try_smaller = 10; //How many micrometres to inset with the try with a smaller inset.
-        if (try_line_thickness && inset_part_count > minimum_part_saving + 1 && (i == 0 || (i > 0 && inset_part_count > part->insets[i - 1].size() + minimum_part_saving)))
+        if (inset_part_count > minimum_part_saving + 1 && (i == 0 || (i > 0 && inset_part_count > part->insets[i - 1].size() + minimum_part_saving)))
         {
             //Try a different line thickness and see if this fits better, based on these criteria:
             // - There are fewer parts to the polygon (fits better in slim areas).
@@ -109,6 +131,7 @@ void WallsComputation::generateInsets(SliceLayer* layer)
         generateInsets(&layer->parts[partNr]);
     }
 
+    const bool remove_parts_with_no_insets = !settings.get<bool>("fill_outline_gaps");
     //Remove the parts which did not generate an inset. As these parts are too small to print,
     // and later code can now assume that there is always minimal 1 inset line.
     for (unsigned int part_idx = 0; part_idx < layer->parts.size(); part_idx++)
