@@ -1,96 +1,112 @@
-/** Copyright (C) 2013 Ultimaker - Released under terms of the AGPLv3 License */
+//Copyright (c) 2019 Ultimaker B.V.
+//CuraEngine is released under the terms of the AGPLv3 or higher.
+
 #ifndef GCODEEXPORT_H
 #define GCODEEXPORT_H
 
-#include <stdio.h>
 #include <deque> // for extrusionAmountAtPreviousRetractions
+#ifdef BUILD_TESTS
+    #include <gtest/gtest_prod.h> //To allow tests to use protected members.
+#endif
 #include <sstream> // for stream.str()
+#include <stdio.h>
 
-#include "settings/settings.h"
-#include "utils/intpoint.h"
-#include "utils/NoCopy.h"
+#include "utils/AABB3D.h" //To track the used build volume for the Griffin header.
 #include "timeEstimate.h"
-#include "MeshGroup.h"
-#include "commandSocket.h"
-#include "RetractionConfig.h"
+#include "settings/EnumSettings.h"
+#include "settings/Settings.h" //For MAX_EXTRUDERS.
+#include "settings/types/Temperature.h" //Bed temperature.
+#include "settings/types/Velocity.h"
+#include "utils/IntPoint.h"
+#include "utils/NoCopy.h"
 
-namespace cura {
-
-/*!
- * Coasting configuration used during printing.
- * Can differ per extruder.
- * 
- * Might be used in the future to have different coasting per feature, e.g. outer wall only.
- */
-struct CoastingConfig
+namespace cura
 {
-    bool coasting_enable; //!< Whether coasting is enabled on the extruder to which this config is attached 
-    double coasting_volume; //!< The volume leeked when printing without feeding
-    double coasting_speed; //!< A modifier (0-1) on the last used travel speed to move slower during coasting
-    double coasting_min_volume;  //!< The minimal volume printed to build up enough pressure to leek the coasting_volume
-};
 
+struct LayerIndex;
+class RetractionConfig;
+struct WipeScriptConfig;
 
 //The GCodeExport class writes the actual GCode. This is the only class that knows how GCode looks and feels.
 //  Any customizations on GCodes flavors are done in this class.
 class GCodeExport : public NoCopy
 {
+#ifdef BUILD_TESTS
+    friend class GCodeExportTest;
+    friend class GriffinHeaderTest;
+    FRIEND_TEST(GCodeExportTest, CommentEmpty);
+    FRIEND_TEST(GCodeExportTest, CommentSimple);
+    FRIEND_TEST(GCodeExportTest, CommentMultiLine);
+    FRIEND_TEST(GCodeExportTest, CommentMultiple);
+    FRIEND_TEST(GCodeExportTest, CommentTimeZero);
+    FRIEND_TEST(GCodeExportTest, CommentTimeInteger);
+    FRIEND_TEST(GCodeExportTest, CommentTimeFloatRoundingError);
+    FRIEND_TEST(GCodeExportTest, CommentTypeAllTypesCovered);
+    FRIEND_TEST(GCodeExportTest, CommentLayer);
+    FRIEND_TEST(GCodeExportTest, CommentLayerNegative);
+    FRIEND_TEST(GCodeExportTest, CommentLayerCount);
+    FRIEND_TEST(GriffinHeaderTest, HeaderGriffinFormat);
+    FRIEND_TEST(GCodeExportTest, HeaderUltiGCode);
+    FRIEND_TEST(GCodeExportTest, HeaderRepRap);
+    FRIEND_TEST(GCodeExportTest, HeaderMarlin);
+    FRIEND_TEST(GCodeExportTest, HeaderMarlinVolumetric);
+    FRIEND_TEST(GCodeExportTest, EVsMmVolumetric);
+    FRIEND_TEST(GCodeExportTest, EVsMmLinear);
+    FRIEND_TEST(GCodeExportTest, WriteZHopStartDefaultSpeed);
+    FRIEND_TEST(GCodeExportTest, WriteZHopStartCustomSpeed);
+    FRIEND_TEST(GCodeExportTest, WriteZHopEndZero);
+    FRIEND_TEST(GCodeExportTest, WriteZHopEndDefaultSpeed);
+    FRIEND_TEST(GCodeExportTest, WriteZHopEndCustomSpeed);
+    FRIEND_TEST(GCodeExportTest, insertWipeScriptSingleMove);
+    FRIEND_TEST(GCodeExportTest, insertWipeScriptMultipleMoves);
+    FRIEND_TEST(GCodeExportTest, insertWipeScriptOptionalDelay);
+    FRIEND_TEST(GCodeExportTest, insertWipeScriptRetractionEnable);
+    FRIEND_TEST(GCodeExportTest, insertWipeScriptHopEnable);
+#endif
 private:
     struct ExtruderTrainAttributes
     {
-        Point3 prime_pos; //!< The location this nozzle is primed before printing
-        bool prime_pos_is_abs; //!< Whether the prime position is absolute, rather than relative to the last given position
         bool is_primed; //!< Whether this extruder has currently already been primed in this print
-        bool use_temp; //!< Whether to insert temperature commands for this extruder
-        bool is_prime_blob_enabled; //! < Whether the priming blob is enabled
 
         bool is_used; //!< Whether this extruder train is actually used during the printing of all meshgroups
-        int nozzle_size; //!< The nozzle size label of the nozzle (e.g. 0.4mm; irrespective of tolerances)
-        Point nozzle_offset;
         char extruderCharacter;
-        std::string material_guid; //!< The GUID for the material used by this extruder
 
-        std::string start_code;
-        std::string end_code;
         double filament_area; //!< in mm^2 for non-volumetric, cylindrical filament
 
         double totalFilament; //!< total filament used per extruder in mm^3
-        int currentTemperature;
-        int initial_temp; //!< Temperature this nozzle needs to be at the start of the print.
+        Temperature currentTemperature;
+        bool waited_for_temperature; //!< Whether the most recent temperature command has been a heat-and-wait command (M109) or not (M104).
+        Temperature initial_temp; //!< Temperature this nozzle needs to be at the start of the print.
 
         double retraction_e_amount_current; //!< The current retracted amount (in mm or mm^3), or zero(i.e. false) if it is not currently retracted (positive values mean retracted amount, so negative impact on E values)
         double retraction_e_amount_at_e_start; //!< The ExtruderTrainAttributes::retraction_amount_current value at E0, i.e. the offset (in mm or mm^3) from E0 to the situation where the filament is at the tip of the nozzle.
 
         double prime_volume; //!< Amount of material (in mm^3) to be primed after an unretration (due to oozing and/or coasting)
-        double last_retraction_prime_speed; //!< The last prime speed (in mm/s) of the to-be-primed amount
+        Velocity last_retraction_prime_speed; //!< The last prime speed (in mm/s) of the to-be-primed amount
 
-        std::string nozzle_id; //!< Type of the printcore, such as "AA 0.4", "BB 0.8", etc.
+        double last_e_value_after_wipe; //!< The current material amount extruded since last wipe
+
+        unsigned fan_number; // nozzle print cooling fan number
 
         std::deque<double> extruded_volume_at_previous_n_retractions; // in mm^3
 
         ExtruderTrainAttributes()
-        : prime_pos(0, 0, 0)
-        , prime_pos_is_abs(false)
-        , is_primed(false)
-        , is_prime_blob_enabled(false)
+        : is_primed(false)
         , is_used(false)
-        , nozzle_offset(0,0)
         , extruderCharacter(0)
-        , start_code("")
-        , end_code("")
         , filament_area(0)
         , totalFilament(0)
         , currentTemperature(0)
+        , waited_for_temperature(false)
         , initial_temp(0)
         , retraction_e_amount_current(0.0)
         , retraction_e_amount_at_e_start(0.0)
         , prime_volume(0.0)
         , last_retraction_prime_speed(0.0)
-        , nozzle_id("")
+        , fan_number(0)
         { }
     };
     ExtruderTrainAttributes extruder_attr[MAX_EXTRUDERS];
-    unsigned int extruder_count;
     bool use_extruder_offset_to_offset_coords;
     std::string machine_name;
     std::string machine_buildplate_type;
@@ -106,11 +122,10 @@ private:
     double extrusion_offset_factor; //!< default 1
 
     Point3 currentPosition; //!< The last build plate coordinates written to gcode (which might be different from actually written gcode coordinates when the extruder offset is encoded in the gcode)
-    double currentSpeed; //!< The current speed (F values / 60) in mm/s
-    double current_print_acceleration; //!< The current acceleration (in mm/s^2) used for print moves (and also for travel moves if the gcode flavor doesn't have separate travel acceleration)
-    double current_travel_acceleration; //!< The current acceleration (in mm/s^2) used for travel moves for those gcode flavors that have separate print and travel accelerations
-    double current_jerk; //!< The current jerk in the XY direction (in mm/s^3)
-    double current_max_z_feedrate; //!< The current max z speed (in mm/s)
+    Velocity currentSpeed; //!< The current speed (F values / 60) in mm/s
+    Acceleration current_print_acceleration; //!< The current acceleration (in mm/s^2) used for print moves (and also for travel moves if the gcode flavor doesn't have separate travel acceleration)
+    Acceleration current_travel_acceleration; //!< The current acceleration (in mm/s^2) used for travel moves for those gcode flavors that have separate print and travel accelerations
+    Velocity current_jerk; //!< The current jerk in the XY direction (in mm/s^3)
 
     AABB3D total_bounding_box; //!< The bounding box of all g-code.
 
@@ -121,23 +136,25 @@ private:
      * 
      * \note After GCodeExport::writeExtrusion(Point, double, double) has been called currentPosition.z coincides with this value
      */
-    int current_layer_z;
-    int isZHopped; //!< The amount by which the print head is currently z hopped, or zero if it is not z hopped. (A z hop is used during travel moves to avoid collision with other layer parts)
+    coord_t current_layer_z;
+    coord_t is_z_hopped; //!< The amount by which the print head is currently z hopped, or zero if it is not z hopped. (A z hop is used during travel moves to avoid collision with other layer parts)
 
-    int current_extruder;
-    int currentFanSpeed;
+    size_t current_extruder;
+    double current_fan_speed;
+    unsigned fan_number; // current print cooling fan number
     EGCodeFlavor flavor;
 
-    std::vector<double> total_print_times; //!< The total estimated print time in seconds for each feature
+    std::vector<Duration> total_print_times; //!< The total estimated print time in seconds for each feature
     TimeEstimateCalculator estimateCalculator;
-    
-    bool is_volumatric;
-    bool firmware_retract; //!< whether retractions are done in the firmware, or hardcoded in E values.
-    bool relative_extrusion; //!< whether to use relative extrusion distances rather than absolute
 
     unsigned int layer_nr; //!< for sending travel data
 
-    int initial_bed_temp; //!< bed temperature at the beginning of the print.
+    bool is_volumetric;
+    bool relative_extrusion; //!< whether to use relative extrusion distances rather than absolute
+
+    Temperature initial_bed_temp; //!< bed temperature at the beginning of the print.
+    Temperature build_volume_temperature;  //!< build volume temperature
+    bool machine_heated_build_volume;  //!< does the machine have the ability to control/stabilize build-volume-temperature
 protected:
     /*!
      * Convert an E value to a value in mm (if it wasn't already in mm) for the current extruder.
@@ -172,11 +189,31 @@ protected:
      */
     double mmToE(double mm);
 
+    /*!
+     * Convert an E value to a value in mm3 (if it wasn't already in mm3) for the provided extruder.
+     *
+     * E values are either in mm or in mm^3
+     * The given extruder is used to determine the filament area to make the conversion.
+     *
+     * \param e the value to convert
+     * \param extruder Extruder number
+     * \return the value converted to mm3
+     */
+    double eToMm3(double e, size_t extruder);
+
 public:
     
     GCodeExport();
     ~GCodeExport();
 
+    /*
+     * \brief Converts the g-code flavor to a string as it must be printed in
+     * the g-code.
+     * \param flavor The g-code flavor to print.
+     * \return A serialized form of this flavor.
+     */
+    const std::string flavorToString(const EGCodeFlavor& flavor) const;
+
     /*!
      * Get the gcode file header (e.g. ";FLAVOR:UltiGCode\n")
      * 
@@ -186,23 +223,15 @@ public:
      * \param mat_ids The material GUIDs for each material.
      * \return The string representing the file header
      */
-    std::string getFileHeader(const std::vector<bool>& extruder_is_used, const double* print_time = nullptr, const std::vector<double>& filament_used = std::vector<double>(), const std::vector<std::string>& mat_ids = std::vector<std::string>());
+    std::string getFileHeader(const std::vector<bool>& extruder_is_used, const Duration* print_time = nullptr, const std::vector<double>& filament_used = std::vector<double>(), const std::vector<std::string>& mat_ids = std::vector<std::string>());
 
     void setLayerNr(unsigned int layer_nr);
-    
+
     void setOutputStream(std::ostream* stream);
 
     bool getExtruderIsUsed(const int extruder_nr) const; //!< return whether the extruder has been used throughout printing all meshgroup up till now
 
-    bool getExtruderUsesTemp(const int extruder_nr) const; //!< Returns whether the extruder with the given index uses temperature control, i.e. whether temperature commands will be included for this extruder
-
-    int getNozzleSize(const int extruder_nr) const;
-
-    Point getExtruderOffset(const int id) const;
-
-    std::string getMaterialGUID(const int extruder_nr) const; //!< returns the GUID of the material used for the nozzle with id \p extruder_nr
-
-    Point getGcodePos(const int64_t x, const int64_t y, const int extruder_train) const;
+    Point getGcodePos(const coord_t x, const coord_t y, const int extruder_train) const;
     
     void setFlavor(EGCodeFlavor flavor);
     EGCodeFlavor getFlavor() const;
@@ -211,9 +240,14 @@ public:
 
     void setFlowRateExtrusionSettings(double max_extrusion_offset, double extrusion_offset_factor);
 
-    void addLastCoastedVolume(double last_coasted_volume) 
+    /*!
+     * Add extra amount of material to be primed after an unretraction.
+     *
+     * \param extra_prime_distance Amount of material in mm.
+     */
+    void addExtraPrimeAmount(double extra_prime_volume)
     {
-        extruder_attr[current_extruder].prime_volume += last_coasted_volume; 
+        extruder_attr[current_extruder].prime_volume += extra_prime_volume;
     }
     
     Point3 getPosition() const;
@@ -224,7 +258,7 @@ public:
 
     int getExtruderNr() const;
     
-    void setFilamentDiameter(unsigned int n, int diameter);
+    void setFilamentDiameter(size_t extruder, const coord_t diameter);
     
     double getCurrentExtrudedVolume() const;
 
@@ -236,14 +270,14 @@ public:
      * \param extruder_nr The extruder number for which to get the total netto extruded volume
      * \return total filament printed in mm^3
      */
-    double getTotalFilamentUsed(int extruder_nr);
+    double getTotalFilamentUsed(size_t extruder_nr);
 
     /*!
      * Get the total estimated print time in seconds for each feature
      * 
      * \return total print time in seconds for each feature
      */
-    std::vector<double> getTotalPrintTimePerFeature();
+    std::vector<Duration> getTotalPrintTimePerFeature();
     /*!
      * Get the total print time in seconds for the complete print
      * 
@@ -253,8 +287,8 @@ public:
     void updateTotalPrintTime();
     void resetTotalPrintTimeAndFilament();
     
-    void writeComment(std::string comment);
-    void writeTypeComment(PrintFeatureType type);
+    void writeComment(const std::string& comment);
+    void writeTypeComment(const PrintFeatureType& type);
 
     /*!
      * Write an M82 (absolute) or M83 (relative)
@@ -268,9 +302,17 @@ public:
      * 
      * \param time The time passed up till this point
      */
-    void writeTimeComment(const double time);
-    void writeLayerComment(int layer_nr);
-    void writeLayerCountComment(int layer_count);
+    void writeTimeComment(const Duration time);
+
+    /*!
+     * Write a comment saying that we're starting a certain layer.
+     */
+    void writeLayerComment(const LayerIndex layer_nr);
+
+    /*!
+     * Write a comment saying that the print has a certain number of layers.
+     */
+    void writeLayerCountComment(const size_t layer_count);
     
     void writeLine(const char* line);
     
@@ -281,7 +323,7 @@ public:
      */
     void resetExtrusionValue();
     
-    void writeDelay(double timeAmount);
+    void writeDelay(const Duration& time_amount);
 
     /*!
      * Coordinates are build plate coordinates, which might be offsetted when extruder offsets are encoded in the gcode.
@@ -289,7 +331,7 @@ public:
      * \param p location to go to
      * \param speed movement speed
      */
-    void writeTravel(Point p, double speed);
+    void writeTravel(const Point& p, const Velocity& speed);
 
     /*!
      * Coordinates are build plate coordinates, which might be offsetted when extruder offsets are encoded in the gcode.
@@ -299,7 +341,7 @@ public:
      * \param feature the feature that's currently printing
      * \param update_extrusion_offset whether to update the extrusion offset to match the current flow rate
      */
-    void writeExtrusion(Point p, double speed, double extrusion_mm3_per_mm, PrintFeatureType feature, bool update_extrusion_offset = false);
+    void writeExtrusion(const Point& p, const Velocity& speed, double extrusion_mm3_per_mm, PrintFeatureType feature, bool update_extrusion_offset = false);
 
     /*!
      * Go to a X/Y location with the z-hopped Z value
@@ -308,7 +350,7 @@ public:
      * \param p location to go to
      * \param speed movement speed
      */
-    void writeTravel(Point3 p, double speed);
+    void writeTravel(const Point3& p, const Velocity& speed);
 
     /*!
      * Go to a X/Y location with the extrusion Z
@@ -322,7 +364,7 @@ public:
      * \param feature the feature that's currently printing
      * \param update_extrusion_offset whether to update the extrusion offset to match the current flow rate
      */
-    void writeExtrusion(Point3 p, double speed, double extrusion_mm3_per_mm, PrintFeatureType feature, bool update_extrusion_offset = false);
+    void writeExtrusion(const Point3& p, const Velocity& speed, double extrusion_mm3_per_mm, PrintFeatureType feature, bool update_extrusion_offset = false);
 private:
     /*!
      * Coordinates are build plate coordinates, which might be offsetted when extruder offsets are encoded in the gcode.
@@ -332,7 +374,7 @@ private:
      * \param z build plate z
      * \param speed movement speed
      */
-    void writeTravel(int x, int y, int z, double speed);
+    void writeTravel(const coord_t& x, const coord_t& y, const coord_t& z, const Velocity& speed);
 
     /*!
      * Perform un-z-hop
@@ -348,7 +390,7 @@ private:
      * \param feature the print feature that's currently printing
      * \param update_extrusion_offset whether to update the extrusion offset to match the current flow rate
      */
-    void writeExtrusion(int x, int y, int z, double speed, double extrusion_mm3_per_mm, PrintFeatureType feature, bool update_extrusion_offset = false);
+    void writeExtrusion(const int x, const int y, const int z, const Velocity& speed, const double extrusion_mm3_per_mm, const PrintFeatureType& feature, const bool update_extrusion_offset = false);
 
     /*!
      * Write the F, X, Y, Z and E value (if they are not different from the last)
@@ -360,7 +402,7 @@ private:
      * It estimates the time in \ref GCodeExport::estimateCalculator for the correct feature
      * It updates \ref GCodeExport::currentPosition, \ref GCodeExport::current_e_value and \ref GCodeExport::currentSpeed
      */
-    void writeFXYZE(double speed, int x, int y, int z, double e, PrintFeatureType feature);
+    void writeFXYZE(const Velocity& speed, const int x, const int y, const int z, const double e, const PrintFeatureType& feature);
 
     /*!
      * The writeTravel and/or writeExtrusion when flavor == BFB
@@ -371,7 +413,7 @@ private:
      * \param extrusion_mm3_per_mm flow
      * \param feature print feature to track print time for
      */
-    void writeMoveBFB(int x, int y, int z, double speed, double extrusion_mm3_per_mm, PrintFeatureType feature);
+    void writeMoveBFB(const int x, const int y, const int z, const Velocity& speed, double extrusion_mm3_per_mm, PrintFeatureType feature);
 public:
     /*!
      * Get ready for extrusion moves:
@@ -385,17 +427,19 @@ public:
     void writeRetraction(const RetractionConfig& config, bool force = false, bool extruder_switch = false);
 
     /*!
-     * Start a z hop with the given \p hop_height
+     * Start a z hop with the given \p hop_height.
      * 
-     * \param hop_height The height to move above the current layer
+     * \param hop_height The height to move above the current layer.
+     * \param speed The speed used for moving. 
      */
-    void writeZhopStart(int hop_height);
+    void writeZhopStart(const coord_t hop_height, Velocity speed = 0);
 
     /*!
      * End a z hop: go back to the layer height
-     * 
+     *
+     * \param speed The speed used for moving.
      */
-    void writeZhopEnd();
+    void writeZhopEnd(Velocity speed = 0);
 
     /*!
      * Start the new_extruder: 
@@ -405,11 +449,11 @@ public:
      * 
      * \param new_extruder The extruder to start with
      */
-    void startExtruder(int new_extruder);
+    void startExtruder(const size_t new_extruder);
 
     /*!
      * Switch to the new_extruder: 
-     * - perform neccesary retractions
+     * - perform neccessary retractions
      * - fiddle with E-values
      * - write extruder end gcode
      * - set new extruder
@@ -417,8 +461,9 @@ public:
      * 
      * \param new_extruder The extruder to switch to
      * \param retraction_config_old_extruder The extruder switch retraction config of the old extruder, to perform the extruder switch retraction with.
+     * \param perform_z_hop The amount by which the print head should be z hopped during extruder switch, or zero if it should not z hop.
      */
-    void switchExtruder(int new_extruder, const RetractionConfig& retraction_config_old_extruder);
+    void switchExtruder(size_t new_extruder, const RetractionConfig& retraction_config_old_extruder, coord_t perform_z_hop = 0);
 
     void writeCode(const char* str);
     
@@ -427,57 +472,49 @@ public:
      * 
      * \param travel_speed The travel speed when priming involves a movement
      */
-    void writePrimeTrain(double travel_speed);
+    void writePrimeTrain(const Velocity& travel_speed);
+
+    /*!
+     * Set the print cooling fan number (used as P parameter to M10[67]) for the specified extruder
+     *
+     * \param extruder The current extruder
+     */
+    void setExtruderFanNumber(int extruder);
     
     void writeFanCommand(double speed);
     
-    void writeTemperatureCommand(int extruder, double temperature, bool wait = false);
-    void writeBedTemperatureCommand(double temperature, bool wait = false);
+    void writeTemperatureCommand(const size_t extruder, const Temperature& temperature, const bool wait = false);
+    void writeBedTemperatureCommand(const Temperature& temperature, const bool wait = false);
+    void writeBuildVolumeTemperatureCommand(const Temperature& temperature, const bool wait = false);
 
     /*!
      * Write the command for setting the acceleration for print moves to a specific value
      */
-    void writePrintAcceleration(double acceleration);
+    void writePrintAcceleration(const Acceleration& acceleration);
 
     /*!
      * Write the command for setting the acceleration for travel moves to a specific value
      */
-    void writeTravelAcceleration(double acceleration);
+    void writeTravelAcceleration(const Acceleration& acceleration);
 
     /*!
      * Write the command for setting the jerk to a specific value
      */
-    void writeJerk(double jerk);
+    void writeJerk(const Velocity& jerk);
 
     /*!
-     * Write the command for setting the maximum z feedrate to a specific value
+     * Set member variables using the settings in \p settings.
      */
-    void writeMaxZFeedrate(double max_z_feedrate);
-
-    /*!
-     * Get the last set max z feedrate value sent in the gcode.
-     * 
-     * Returns a value <= 0 when no value is set.
-     */
-    double getCurrentMaxZFeedrate();
-
-    /*!
-     * Set member variables using the settings in \p settings
-     * 
-     * \param settings The meshgroup to get the global bed temp from and to get the extruder trains from which to get the nozzle temperatures
-     */
-    void preSetup(const MeshGroup* settings);
+    void preSetup(const size_t start_extruder);
 
     /*!
      * Handle the initial (bed/nozzle) temperatures before any gcode is processed.
      * These temperatures are set in the pre-print setup in the firmware.
      * 
      * See FffGcodeWriter::processStartingCode
-     * 
-     * \param settings The meshgroup to get the global bed temp from and to get the extruder trains from which to get the nozzle temperatures
      * \param start_extruder_nr The extruder with which to start this print
      */
-    void setInitialTemps(const MeshGroup& settings, const unsigned int start_extruder_nr);
+    void setInitialAndBuildVolumeTemps(const unsigned int start_extruder_nr);
 
     /*!
      * Override or set an initial nozzle temperature as written by GCodeExport::setInitialTemps
@@ -498,6 +535,26 @@ public:
      */
     void finalize(const char* endCode);
 
+    /*!
+     * Get amount of material extruded since last wipe script was inserted.
+     *
+     * \param extruder Extruder number to check.
+     */
+    double getExtrudedVolumeAfterLastWipe(size_t extruder);
+
+    /*!
+     *  Reset the last_e_value_after_wipe.
+     *
+     * \param extruder Extruder number which last_e_value_after_wipe value to reset.
+     */
+     void ResetLastEValueAfterWipe(size_t extruder);
+
+    /*!
+     *  Generate g-code for wiping current nozzle using provided config.
+     *
+     * \param wipe_config Config with wipe script settings.
+     */
+    void insertWipeScript(const WipeScriptConfig& wipe_config);
 };
 
 }
