@@ -1,45 +1,47 @@
-//Copyright (c) 2013 Ultimaker
-//Copyright (c) 2017 Ultimaker B.V.
+//Copyright (c) 2018 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
 #ifndef INFILL_H
 #define INFILL_H
 
-#include "utils/polygon.h"
-#include "settings/settings.h"
 #include "infill/ZigzagConnectorProcessor.h"
-#include "infill/NoZigZagConnectorProcessor.h"
-#include "infill/SubDivCube.h"
-#include "infill/SpaceFillingTreeFill.h"
-#include "utils/intpoint.h"
-#include "utils/AABB.h"
+#include "settings/EnumSettings.h" //For infill types.
+#include "settings/types/AngleDegrees.h"
+#include "utils/IntPoint.h"
 
 namespace cura
 {
 
+class AABB;
+class SierpinskiFillProvider;
+class SliceMeshStorage;
+
 class Infill 
 {
+    friend class InfillTest;
+
     static constexpr int perimeter_gaps_extra_offset = 15; // extra offset so that the perimeter gaps aren't created everywhere due to rounding errors
 
     EFillMethod pattern; //!< the space filling pattern of the infill to generate
     bool zig_zaggify; //!< Whether to connect the end pieces of the support lines via the wall
+    bool connect_polygons; //!< Whether to connect as much polygons together into a single path
     const Polygons& in_outline; //!< a reference polygon for getting the actual area within which to generate infill (see outline_offset)
     coord_t outline_offset; //!< Offset from Infill::in_outline to get the actual area within which to generate infill
     coord_t infill_line_width; //!< The line width of the infill lines to generate
     coord_t line_distance; //!< The distance between two infill lines / polygons
     coord_t infill_overlap; //!< the distance by which to overlap with the actual area within which to generate infill
-    double fill_angle; //!< for linear infill types: the angle of the infill lines (or the angle of the grid)
+    size_t infill_multiplier; //!< the number of infill lines next to each other
+    AngleDegrees fill_angle; //!< for linear infill types: the angle of the infill lines (or the angle of the grid)
     coord_t z; //!< height of the layer for which we generate infill
     coord_t shift; //!< shift of the scanlines in the direction perpendicular to the fill_angle
+    size_t wall_line_count; //!< Number of walls to generate at the boundary of the infill region, spaced \ref infill_line_width apart
     const Point infill_origin; //!< origin of the infill pattern
     Polygons* perimeter_gaps; //!< (optional output) The areas in between consecutive insets when Concentric infill is used.
     bool connected_zigzags; //!< (ZigZag) Whether endpieces of zigzag infill should be connected to the nearest infill line on both sides of the zigzag connector
     bool use_endpieces; //!< (ZigZag) Whether to include endpieces: zigzag connector segments from one infill line to itself
     bool skip_some_zags;  //!< (ZigZag) Whether to skip some zags
-    int zag_skip_count;  //!< (ZigZag) To skip one zag in every N if skip some zags is enabled
-    bool apply_pockets_alternatingly; //!< Whether to add pockets to the cross 3d pattern only at half the intersections of the fractal
+    size_t zag_skip_count;  //!< (ZigZag) To skip one zag in every N if skip some zags is enabled
     coord_t pocket_size; //!< The size of the pockets at the intersections of the fractal in the cross 3d pattern
-    coord_t minimum_zag_line_length; //!< Throw away perimeters that are too small
 
     static constexpr double one_over_sqrt_2 = 0.7071067811865475244008443621048490392848359376884740; //!< 1.0 / sqrt(2.0)
 public:
@@ -53,43 +55,45 @@ public:
      */
     Infill(EFillMethod pattern
         , bool zig_zaggify
+        , bool connect_polygons
         , const Polygons& in_outline
-        , int outline_offset
-        , int infill_line_width
-        , int line_distance
-        , int infill_overlap
-        , double fill_angle
-        , int64_t z
-        , int64_t shift
+        , coord_t outline_offset
+        , coord_t infill_line_width
+        , coord_t line_distance
+        , coord_t infill_overlap
+        , size_t infill_multiplier
+        , AngleDegrees fill_angle
+        , coord_t z
+        , coord_t shift
+        , size_t wall_line_count = 0
         , const Point& infill_origin = Point()
         , Polygons* perimeter_gaps = nullptr
         , bool connected_zigzags = false
         , bool use_endpieces = false
         , bool skip_some_zags = false
-        , int zag_skip_count = 0
-        , bool apply_pockets_alternatingly = false
+        , size_t zag_skip_count = 0
         , coord_t pocket_size = 0
-        , coord_t minimum_zag_line_length = DEFAULT_MINIMUM_LINE_LENGTH_THRESHOLD
     )
     : pattern(pattern)
     , zig_zaggify(zig_zaggify)
+    , connect_polygons(connect_polygons)
     , in_outline(in_outline)
     , outline_offset(outline_offset)
     , infill_line_width(infill_line_width)
     , line_distance(line_distance)
     , infill_overlap(infill_overlap)
+    , infill_multiplier(infill_multiplier)
     , fill_angle(fill_angle)
     , z(z)
     , shift(shift)
+    , wall_line_count(wall_line_count)
     , infill_origin(infill_origin)
     , perimeter_gaps(perimeter_gaps)
     , connected_zigzags(connected_zigzags)
     , use_endpieces(use_endpieces)
     , skip_some_zags(skip_some_zags)
     , zag_skip_count(zag_skip_count)
-    , apply_pockets_alternatingly(apply_pockets_alternatingly)
     , pocket_size(pocket_size)
-    , minimum_zag_line_length(minimum_zag_line_length)
     {
     }
 
@@ -99,11 +103,29 @@ public:
      * \param result_polygons (output) The resulting polygons (from concentric infill)
      * \param result_lines (output) The resulting line segments (from linear infill types)
      * \param mesh The mesh for which to generate infill (should only be used for non-helper objects)
-     * \param[in] cross_fill_pattern Where the cross fractal precomputation is stored
+     * \param[in] cross_fill_provider The cross fractal subdivision decision functor
      */
-    void generate(Polygons& result_polygons, Polygons& result_lines, const SpaceFillingTreeFill* cross_fill_pattern = nullptr, const SliceMeshStorage* mesh = nullptr);
+    void generate(Polygons& result_polygons, Polygons& result_lines, const SierpinskiFillProvider* cross_fill_provider = nullptr, const SliceMeshStorage* mesh = nullptr);
 
 private:
+    /*!
+     * Generate the infill pattern without the infill_multiplier functionality
+     */
+    void _generate(Polygons& result_polygons, Polygons& result_lines, const SierpinskiFillProvider* cross_fill_pattern = nullptr, const SliceMeshStorage* mesh = nullptr);
+
+    /*!
+     * Multiply the infill lines, so that any single line becomes [infill_multiplier] lines next to each other.
+     * 
+     * This is done in a way such that there is not overlap between the lines
+     * except the middle original one if the multiplier is odd.
+     * 
+     * This introduces a lot of line segments.
+     * 
+     * \param[in,out] result_polygons The polygons to be multiplied (input and output)
+     * \param[in,out] result_lines The lines to be multiplied (input and output)
+     */
+    void multiplyInfill(Polygons& result_polygons, Polygons& result_lines);
+
     struct InfillLineSegment
     {
         /*!
@@ -197,6 +219,12 @@ private:
      */
     std::vector<std::vector<std::vector<InfillLineSegment*>>> crossings_on_line;
 
+    /*!
+     * Generate gyroid infill
+     * \param result (output) The resulting polygons
+     */
+    void generateGyroidInfill(Polygons& result);
+    
     /*!
      * Generate sparse concentric infill
      * 
@@ -215,12 +243,6 @@ private:
      */
     void generateConcentricInfill(Polygons& first_wall, Polygons& result, int inset_value);
 
-    /*!
-     * Generate sparse concentric infill 
-     * \param[out] result (output) The resulting lines
-     */
-    void generateConcentric3DInfill(Polygons& result);
-
     /*!
      * Generate a rectangular grid of infill lines
      * \param[out] result (output) The resulting lines
@@ -276,11 +298,11 @@ private:
 
     /*!
      * Generate a 3d pattern of subdivided cubes on their points
-     * \param[in] cross_fill_pattern Where the cross fractal precomputation is stored
+     * \param[in] cross_fill_provider Where the cross fractal precomputation is stored
      * \param[out] result_polygons The resulting polygons
      * \param[out] result_lines The resulting lines
      */
-    void generateCrossInfill(const SpaceFillingTreeFill& cross_fill_pattern, Polygons& result_polygons, Polygons& result_lines);
+    void generateCrossInfill(const SierpinskiFillProvider& cross_fill_provider, Polygons& result_polygons, Polygons& result_lines);
 
     /*!
      * Convert a mapping from scanline to line_segment-scanline-intersections (\p cut_list) into line segments, using the even-odd rule
@@ -292,7 +314,7 @@ private:
      * \param cut_list A mapping of each scanline to all y-coordinates (in the space transformed by rotation_matrix) where the polygons are crossing the scanline
      * \param total_shift total shift of the scanlines in the direction perpendicular to the fill_angle.
      */
-    void addLineInfill(Polygons& result, const PointMatrix& rotation_matrix, const int scanline_min_idx, const int line_distance, const AABB boundary, std::vector<std::vector<int64_t>>& cut_list, int64_t total_shift);
+    void addLineInfill(Polygons& result, const PointMatrix& rotation_matrix, const int scanline_min_idx, const int line_distance, const AABB boundary, std::vector<std::vector<coord_t>>& cut_list, coord_t total_shift);
 
     /*!
      * Crop line segments by the infill polygon using Clipper
@@ -312,7 +334,7 @@ private:
      * \param infill_rotation The angle of the generated lines
      * \param extra_shift extra shift of the scanlines in the direction perpendicular to the infill_rotation
      */
-    void generateLineInfill(Polygons& result, int line_distance, const double& infill_rotation, int64_t extra_shift);
+    void generateLineInfill(Polygons& result, int line_distance, const double& infill_rotation, coord_t extra_shift);
     
     /*!
      * Function for creating linear based infill types (Lines, ZigZag).
@@ -330,7 +352,7 @@ private:
      * \param connected_zigzags Whether to connect the endpiece zigzag segments on both sides to the same infill line
      * \param extra_shift extra shift of the scanlines in the direction perpendicular to the fill_angle
      */
-    void generateLinearBasedInfill(const int outline_offset, Polygons& result, const int line_distance, const PointMatrix& rotation_matrix, ZigzagConnectorProcessor& zigzag_connector_processor, const bool connected_zigzags, int64_t extra_shift);
+    void generateLinearBasedInfill(const int outline_offset, Polygons& result, const int line_distance, const PointMatrix& rotation_matrix, ZigzagConnectorProcessor& zigzag_connector_processor, const bool connected_zigzags, coord_t extra_shift);
 
     /*!
      * 
@@ -379,7 +401,7 @@ private:
      * \param line_distance The distance between two lines which are in the same direction
      * \param infill_rotation The angle of the generated lines
      */
-    void generateZigZagInfill(Polygons& result, const int line_distance, const double& infill_rotation);
+    void generateZigZagInfill(Polygons& result, const coord_t line_distance, const double& infill_rotation);
 
     /*!
      * determine how far the infill pattern should be shifted based on the values of infill_origin and \p infill_rotation
@@ -388,7 +410,7 @@ private:
      *
      * \return the distance the infill pattern should be shifted
      */
-    int64_t getShiftOffsetFromInfillOriginAndRotation(const double& infill_rotation);
+    coord_t getShiftOffsetFromInfillOriginAndRotation(const double& infill_rotation);
 
     /*!
      * Connects infill lines together so that they form polylines.
