@@ -1,17 +1,20 @@
-/** Copyright (C) 2017 Ultimaker - Released under terms of the AGPLv3 License */
+//Copyright (c) 2018 Ultimaker B.V.
+//CuraEngine is released under the terms of the AGPLv3 or higher.
+
 #ifndef INFILL_SPAGHETTI_INFILL_PATH_GENERATOR_H
 #define INFILL_SPAGHETTI_INFILL_PATH_GENERATOR_H
 
 #include <list>
+#include "../settings/PathConfigStorage.h" //For the MeshPathConfigs subclass.
 
-#include "../utils/intpoint.h"
-#include "../utils/polygon.h"
-#include "../sliceDataStorage.h"
-#include "../LayerPlan.h"
-
-namespace cura {
+namespace cura
+{
 
 class FffGcodeWriter;
+class LayerPlan;
+class SliceDataStorage;
+class SliceLayerPart;
+class SliceMeshStorage;
 
 /*!
  * Spaghetti infill is a type of infill which fills every so many layers, but extrudes as much filament corresponding to the total unfilled volume under the filling area.
@@ -29,29 +32,26 @@ class FffGcodeWriter;
 class SpaghettiInfillPathGenerator
 {
 public:
-
     /*!
      * Add spaghetti infill for a given part in a layer plan.
      * 
      * Move over the infill region with a zigzag pattern and
      * extrude as much material as needed for the current part
-     * and all parts below which should be filled withthis spaghetti.
+     * and all parts below which should be filled with this spaghetti.
      * 
-     * \param storage Where to get the secondary brim lines from if we are adding spaghetti on the very first layer and we need to prime first
-     * \param fff_gcode_writer The FffGcodeWriter for which the function setExtruder_addPrime needs to be called when on the first layer
+     * \param storage Where to get the secondary brim lines from if we are
+     * adding spaghetti on the very first layer and we need to prime first.
+     * \param fff_gcode_writer The FffGcodeWriter for which the function
+     * setExtruder_addPrime needs to be called when on the first layer.
      * \param gcode_layer The initial planning of the gcode of the layer.
      * \param mesh The mesh for which to add to the layer plan \p gcode_layer.
-     * \param extruder_nr The extruder for which to print all features of the mesh which should be printed with this extruder
-     * \param mesh_config the line config with which to print a print feature
-     * \param part The part for which to create gcode
-     * \param infill_line_distance The distance between the infill lines
-     * \param infill_overlap The distance by which the infill overlaps with the wall insets.
-     * \param fillAngle The angle in the XY plane at which the infill is generated.
-     * \param infill_origin The origin of the infill pattern.
+     * \param extruder_nr The extruder for which to print all features of the
+     * mesh which should be printed with this extruder.
+     * \param mesh_config the line config with which to print a print feature.
+     * \param part The part for which to create gcode.
      * \return Whether this function added anything to the layer plan
      */
-    static bool processSpaghettiInfill(const SliceDataStorage& storage, const FffGcodeWriter& fff_gcode_writer, LayerPlan& gcode_layer, const SliceMeshStorage& mesh, const int extruder_nr, const PathConfigStorage::MeshPathConfigs& mesh_config, const SliceLayerPart& part, int infill_line_distance, int infill_overlap, int fillAngle, const Point& infill_origin);
-
+    static bool processSpaghettiInfill(const SliceDataStorage& storage, const FffGcodeWriter& fff_gcode_writer, LayerPlan& gcode_layer, const SliceMeshStorage& mesh, const size_t extruder_nr, const PathConfigStorage::MeshPathConfigs& mesh_config, const SliceLayerPart& part);
 };
 
 }//namespace cura
