@@ -1,20 +1,26 @@
-//Copyright (c) 2017 Ultimaker B.V.
+//Copyright (c) 2018 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
+#include "Application.h"
 #include "multiVolumes.h"
+#include "Slice.h"
+#include "slicer.h"
+#include "settings/EnumSettings.h"
 
 namespace cura 
 {
  
-void carveMultipleVolumes(std::vector<Slicer*> &volumes, bool alternate_carve_order)
+void carveMultipleVolumes(std::vector<Slicer*> &volumes)
 {
     //Go trough all the volumes, and remove the previous volume outlines from our own outline, so we never have overlapped areas.
+    const bool alternate_carve_order = Application::getInstance().current_slice->scene.current_mesh_group->settings.get<bool>("alternate_carve_order");
     for (unsigned int volume_1_idx = 1; volume_1_idx < volumes.size(); volume_1_idx++)
     {
         Slicer& volume_1 = *volumes[volume_1_idx];
-        if (volume_1.mesh->getSettingBoolean("infill_mesh") 
-            || volume_1.mesh->getSettingBoolean("anti_overhang_mesh")
-            || volume_1.mesh->getSettingBoolean("support_mesh")
+        if (volume_1.mesh->settings.get<bool>("infill_mesh") 
+            || volume_1.mesh->settings.get<bool>("anti_overhang_mesh")
+            || volume_1.mesh->settings.get<bool>("support_mesh")
+            || volume_1.mesh->settings.get<ESurfaceMode>("magic_mesh_surface_mode") == ESurfaceMode::SURFACE
             )
         {
             continue;
@@ -22,9 +28,10 @@ void carveMultipleVolumes(std::vector<Slicer*> &volumes, bool alternate_carve_or
         for (unsigned int volume_2_idx = 0; volume_2_idx < volume_1_idx; volume_2_idx++)
         {
             Slicer& volume_2 = *volumes[volume_2_idx];
-            if (volume_2.mesh->getSettingBoolean("infill_mesh")
-                || volume_2.mesh->getSettingBoolean("anti_overhang_mesh")
-                || volume_2.mesh->getSettingBoolean("support_mesh")
+            if (volume_2.mesh->settings.get<bool>("infill_mesh")
+                || volume_2.mesh->settings.get<bool>("anti_overhang_mesh")
+                || volume_2.mesh->settings.get<bool>("support_mesh")
+                || volume_2.mesh->settings.get<ESurfaceMode>("magic_mesh_surface_mode") == ESurfaceMode::SURFACE
                 )
             {
                 continue;
@@ -62,10 +69,10 @@ void generateMultipleVolumesOverlap(std::vector<Slicer*> &volumes)
     int offset_to_merge_other_merged_volumes = 20;
     for (Slicer* volume : volumes)
     {
-        int overlap = volume->mesh->getSettingInMicrons("multiple_mesh_overlap");
-        if (volume->mesh->getSettingBoolean("infill_mesh")
-            || volume->mesh->getSettingBoolean("anti_overhang_mesh")
-            || volume->mesh->getSettingBoolean("support_mesh")
+        coord_t overlap = volume->mesh->settings.get<coord_t>("multiple_mesh_overlap");
+        if (volume->mesh->settings.get<bool>("infill_mesh")
+            || volume->mesh->settings.get<bool>("anti_overhang_mesh")
+            || volume->mesh->settings.get<bool>("support_mesh")
             || overlap == 0)
         {
             continue;
@@ -77,9 +84,9 @@ void generateMultipleVolumesOverlap(std::vector<Slicer*> &volumes)
             Polygons all_other_volumes;
             for (Slicer* other_volume : volumes)
             {
-                if (other_volume->mesh->getSettingBoolean("infill_mesh")
-                    || other_volume->mesh->getSettingBoolean("anti_overhang_mesh")
-                    || other_volume->mesh->getSettingBoolean("support_mesh")
+                if (other_volume->mesh->settings.get<bool>("infill_mesh")
+                    || other_volume->mesh->settings.get<bool>("anti_overhang_mesh")
+                    || other_volume->mesh->settings.get<bool>("support_mesh")
                     || !other_volume->mesh->getAABB().hit(aabb)
                     || other_volume == volume
                 )
@@ -101,7 +108,7 @@ void MultiVolumes::carveCuttingMeshes(std::vector<Slicer*>& volumes, const std::
     for (unsigned int carving_mesh_idx = 0; carving_mesh_idx < volumes.size(); carving_mesh_idx++)
     {
         const Mesh& cutting_mesh = meshes[carving_mesh_idx];
-        if (!cutting_mesh.getSettingBoolean("cutting_mesh"))
+        if (!cutting_mesh.settings.get<bool>("cutting_mesh"))
         {
             continue;
         }
@@ -113,8 +120,9 @@ void MultiVolumes::carveCuttingMeshes(std::vector<Slicer*>& volumes, const std::
             for (unsigned int carved_mesh_idx = 0; carved_mesh_idx < volumes.size(); carved_mesh_idx++)
             {
                 const Mesh& carved_mesh = meshes[carved_mesh_idx];
-                //Do not apply cutting_mesh for meshes which have settings (cutting_mesh, anti_overhang_mesh).
-                if (carved_mesh.getSettingBoolean("cutting_mesh") || carved_mesh.getSettingBoolean("anti_overhang_mesh"))
+                //Do not apply cutting_mesh for meshes which have settings (cutting_mesh, anti_overhang_mesh, support_mesh).
+                if (carved_mesh.settings.get<bool>("cutting_mesh") || carved_mesh.settings.get<bool>("anti_overhang_mesh")
+                    || carved_mesh.settings.get<bool>("support_mesh"))
                 {
                     continue;
                 }
