@@ -1,11 +1,13 @@
-/** Copyright (C) 2013 Ultimaker - Released under terms of the AGPLv3 License */
+//Copyright (c) 2018 Ultimaker B.V.
+//CuraEngine is released under the terms of the AGPLv3 or higher.
+
 #ifndef PATHOPTIMIZER_H
 #define PATHOPTIMIZER_H
 
 #include <stdint.h>
+#include "settings/EnumSettings.h"
 #include "utils/polygon.h"
 #include "utils/polygonUtils.h"
-#include "settings/settings.h"
 
 namespace cura {
 
@@ -60,18 +62,20 @@ public:
 
     void addPolygon(PolygonRef polygon)
     {
-        this->polygons.emplace_back(polygon);
+        polygons.emplace_back(polygon);
     }
 
     void addPolygon(ConstPolygonRef polygon)
     {
-        this->polygons.emplace_back(polygon);
+        polygons.emplace_back(polygon);
     }
 
     void addPolygons(const Polygons& polygons)
     {
         for(unsigned int i = 0; i < polygons.size(); i++)
+        {
             this->polygons.emplace_back(polygons[i]);
+        }
     }
 
     void optimize(); //!< sets #polyStart and #polyOrder
@@ -102,18 +106,20 @@ public:
 
     void addPolygon(PolygonRef polygon)
     {
-        this->polygons.push_back(polygon);
+        polygons.push_back(polygon);
     }
 
     void addPolygon(ConstPolygonRef polygon)
     {
-        this->polygons.push_back(polygon);
+        polygons.push_back(polygon);
     }
 
     void addPolygons(Polygons& polygons)
     {
         for(unsigned int i=0;i<polygons.size(); i++)
+        {
             this->polygons.push_back(polygons[i]);
+        }
     }
 
     /*!
@@ -129,33 +135,13 @@ private:
     /*!
      * Update LineOrderOptimizer::polyStart if the current line is better than the current best.
      * 
-     * Besides looking at the distance from the previous line segment, we also look at the angle we make.
-     * 
-     * We prefer 90 degree angles; 180 degree turn arounds are slow on machines where the jerk is limited.
-     * 0 degree (straight ahead) 'corners' occur only when a single infill line is interrupted, 
-     * in which case the travel move might involve combing, which makes it rather longer.
-     * 
      * \param poly_idx[in] The index in LineOrderOptimizer::polygons for the current line to test
      * \param best[in, out] The index of current best line
      * \param best_score[in, out] The distance score for the current best line
      * \param prev_point[in] The previous point from which to find the next best line
-     * \param incoming_perpundicular_normal[in] The direction of movement when the print head arrived at \p prev_point, turned 90 degrees CCW
      * \param just_point[in] If not -1, only look at the line vertex with this index
      */
-    void updateBestLine(unsigned int poly_idx, int& best, float& best_score, Point prev_point, Point incoming_perpundicular_normal, int just_point = -1);
-
-    /*!
-     * Get a score to modify the distance score for measuring how good two lines follow each other.
-     * 
-     * The angle score is symmetric in \p from and \p to; they can be exchanged without altering the result. (Code relies on this property)
-     * 
-     * \param incoming_perpundicular_normal The direction in which the head was moving while printing the previous line, turned 90 degrees CCW
-     * \param from The one end of the next line
-     * \param to The other end of the next line
-     * \return A score measuring how good the angle is of the line between \p from and \p to when the previous line had a direction given by \p incoming_perpundicular_normal 
-     * 
-     */
-    static float getAngleScore(Point incoming_perpundicular_normal, Point from, Point to);
+    void updateBestLine(unsigned int poly_idx, int& best, float& best_score, Point prev_point, int just_point = -1);
 
     /*!
      * Compute the squared distance from \p p0 to \p p1 using combing
