@@ -1,12 +1,20 @@
-/** Copyright (C) 2013 Ultimaker - Released under terms of the AGPLv3 License */
+//Copyright (c) 2018 Ultimaker B.V.
+//CuraEngine is released under the terms of the AGPLv3 or higher.
+
 #ifndef SKIN_H
 #define SKIN_H
 
-#include "sliceDataStorage.h"
+#include "settings/types/LayerIndex.h"
+#include "utils/Coord_t.h"
 
 namespace cura 
 {
 
+class Polygons;
+class SkinPart;
+class SliceLayerPart;
+class SliceMeshStorage;
+
 /*!
  * Class containing all skin and infill area computation functions
  */
@@ -15,13 +23,16 @@ class SkinInfillAreaComputation
 public:
 
     /*!
-     * Initialize the parameters for skin and infill area computation.
+     * \brief Initialize the parameters for skin and infill area computation.
      * 
-     * \param layer_nr The index of the layer for which to generate the skins and infill.
-     * \param mesh The storage where the layer outline information (input) is stored and where the skin insets and fill areas (output) are stored.
-     * \param process_infill Whether to process infill, i.e. whether there's a positive infill density or there are infill meshes modifying this mesh.
+     * \param layer_nr The index of the layer for which to generate the skins
+     * and infill.
+     * \param mesh The storage where the layer outline information (input) is
+     * stored and where the skin insets and fill areas (output) are stored.
+     * \param process_infill Whether to process infill, i.e. whether there's a
+     * positive infill density or there are infill meshes modifying this mesh.
      */
-    SkinInfillAreaComputation(int layer_nr, const SliceDataStorage& storage, SliceMeshStorage& mesh, bool process_infill);
+    SkinInfillAreaComputation(const LayerIndex& layer_nr, SliceMeshStorage& mesh, bool process_infill);
 
     /*!
      * Generate the skin areas and its insets.
@@ -36,24 +47,35 @@ public:
      * all combined layers.
      * 
      * \param mesh The mesh to combine the infill layers of.
-     * \param amount The number of layers to combine.
      */
-    static void combineInfillLayers(SliceMeshStorage& mesh, unsigned int amount);
+    static void combineInfillLayers(SliceMeshStorage& mesh);
 
     /*!
-     * Generate infill areas which cause a gradually less dense infill structure from top to bottom.
-     * 
-     * The areas generated overlap, so that more dense infill adds on to less dense infill.
-     * That way you don't have infill lines which are broken when they cross a border between separated infill areas - if they would be as such.
-     * 
-     * This function also guarantees that the SliceLayerPart::infill_area_per_combine_per_density is initialized with at least one item.
-     * The last item in the list will be equal to the infill_area after this function.
-     * 
-     * \param gradual_infill_step_height // The height difference between consecutive density infill areas
-     * \param max_infill_steps the maximum exponent of division of infill density. At 5 the least dense infill will be 2^4 * infill_line_distance i.e. one 16th as dense
+     * \brief Generate infill areas which cause a gradually less dense infill
+     * structure from top to bottom.
+     *
+     * The areas generated overlap, so that more dense infill adds on to less
+     * dense infill. That way you don't have infill lines which are broken when
+     * they cross a border between separated infill areas, if they would be as
+     * such.
+     *
+     * This function also guarantees that the
+     * SliceLayerPart::infill_area_per_combine_per_density is initialised with
+     * at least one item. The last item in the list will be equal to the
+     * infill_area after this function.
+     * \param mesh The mesh to generate the infill areas for.
      */
-    static void generateGradualInfill(SliceMeshStorage& mesh, unsigned int gradual_infill_step_height, unsigned int max_infill_steps);
+    static void generateGradualInfill(SliceMeshStorage& mesh);
 
+    /*!
+     * Limit the infill areas to places where they support internal overhangs.
+     * 
+     * This function uses the part.infill_area and part.infill_area_own
+     * and computes a new part.infill_area_own
+     * 
+     * \param mesh The mesh for which to recalculate the infill areas
+     */
+    static void generateInfillSupport(SliceMeshStorage& mesh);
 protected:
     /*!
      * Generate the skin areas (outlines) and the infill areas
@@ -68,24 +90,22 @@ protected:
     void generateSkinAndInfillAreas(SliceLayerPart& part);
 
     /*!
-     * Calculate the basic areas which have air above
-     * 
-     * \param part The part for which to compute the top skin areas
-     * \param min_infill_area The minimum area to fill with skin
-     * \param wall_idx The 1-based wall index for the walls to grab. e.g. the outermost walls or the second walls. Zero means the outline.
-     * \param[in,out] upskin The areas of top skin to be pdated by the layers above. The input is the area within the inner walls (or an empty Polygons object).
+     * \brief Calculate the basic areas which have air above.
+     * \param part The part for which to compute the top skin areas.
+     * \param[in,out] upskin The areas of top skin to be updated by the layers
+     * above. The input is the area within the inner walls (or an empty Polygons
+     * object).
      */
-    void calculateTopSkin(const SliceLayerPart& part, int min_infill_area, Polygons& upskin);
+    void calculateTopSkin(const SliceLayerPart& part, Polygons& upskin);
 
     /*!
-     * Calculate the basic areas which have air below
-     * 
-     * \param part The part for which to compute the bottom skin areas
-     * \param min_infill_area The minimum area to fill with skin
-     * \param wall_idx The 1-based wall index for the walls to grab. e.g. the outermost walls or the second walls. Zero means the outline.
-     * \param[in,out] downskin The areas of bottom skin to be updated by the layers above. The input is the area within the inner walls (or an empty Polygons object).
+     * \brief Calculate the basic areas which have air below.
+     * \param part The part for which to compute the bottom skin areas.
+     * \param[in,out] downskin The areas of bottom skin to be updated by the
+     * layers above. The input is the area within the inner walls (or an empty
+     * Polygons object).
      */
-    void calculateBottomSkin(const SliceLayerPart& part, int min_infill_area, Polygons& downskin);
+    void calculateBottomSkin(const SliceLayerPart& part, Polygons& downskin);
 
     /*!
      * Apply skin expansion:
@@ -139,17 +159,18 @@ protected:
     void generateInnerSkinInfill(SkinPart& skin_part);
 
 protected:
-    const int layer_nr; //!< The index of the layer for which to generate the skins and infill.
+    const LayerIndex layer_nr; //!< The index of the layer for which to generate the skins and infill.
     SliceMeshStorage& mesh; //!< The storage where the layer outline information (input) is stored and where the skin insets and fill areas (output) are stored.
-    const int bottom_layer_count; //!< The number of layers of bottom skin
-    const int top_layer_count; //!< The number of layers of top skin
-    const int wall_line_count; //!< The number of walls, i.e. the number of the wall from which to offset.
-    const int skin_line_width; //!< The line width of the skin.
-    const int wall_line_width_0; //!< The line width of the outer wall
-    const int wall_line_width_x; //!< The line width of the inner most wall
-    const int innermost_wall_line_width; //!< width of the innermost wall lines
-    const int infill_skin_overlap; //!< overlap distance between infill and skin
-    const int skin_inset_count; //!< The number of perimeters to surround the skin
+    const size_t bottom_layer_count; //!< The number of layers of bottom skin
+    const size_t initial_bottom_layer_count; //!< Whether to make bottom skin for the initial layer
+    const size_t top_layer_count; //!< The number of layers of top skin
+    const size_t wall_line_count; //!< The number of walls, i.e. the number of the wall from which to offset.
+    const coord_t skin_line_width; //!< The line width of the skin.
+    const coord_t wall_line_width_0; //!< The line width of the outer wall
+    const coord_t wall_line_width_x; //!< The line width of the inner most wall
+    const coord_t innermost_wall_line_width; //!< width of the innermost wall lines
+    const coord_t infill_skin_overlap; //!< overlap distance between infill and skin
+    const size_t skin_inset_count; //!< The number of perimeters to surround the skin
     const bool no_small_gaps_heuristic; //!< A heuristic which assumes there will be no small gaps between bottom and top skin with a z size smaller than the skin size itself
     const bool process_infill; //!< Whether to process infill, i.e. whether there's a positive infill density or there are infill meshes modifying this mesh.
 
@@ -157,13 +178,13 @@ protected:
     coord_t bottom_reference_wall_expansion; //!< The horizontal expansion to apply to the bottom reference wall in order to shrink the bottom skin
     coord_t top_skin_expand_distance; //!< The distance by which the top skins should be larger than the original top skins.
     coord_t bottom_skin_expand_distance; //!< The distance by which the bottom skins should be larger than the original bottom skins.
-    const int top_reference_wall_idx; //!< The wall of the layer above to consider as inside. Lower index means more skin.
-    const int bottom_reference_wall_idx; //!< The wall of the layer below to consider as inside. Lower index means more skin.
+    const size_t top_reference_wall_idx; //!< The wall of the layer above to consider as inside. Lower index means more skin.
+    const size_t bottom_reference_wall_idx; //!< The wall of the layer below to consider as inside. Lower index means more skin.
 private:
-    static coord_t getSkinLineWidth(const SliceDataStorage& storage, const SliceMeshStorage& mesh, int layer_nr); //!< Compute the skin line width, which might be different for the first layer.
-    static coord_t getWallLineWidth0(const SliceDataStorage& storage, const SliceMeshStorage& mesh, int layer_nr); //!< Compute the outer wall line width, which might be different for the first layer
-    static coord_t getWallLineWidthX(const SliceDataStorage& storage, const SliceMeshStorage& mesh, int layer_nr); //!< Compute the inner wall line widths, which might be different for the first layer
-    static coord_t getInfillSkinOverlap(const SliceDataStorage& storage, const SliceMeshStorage& mesh, int layer_nr, coord_t innermost_wall_line_width); //!< Compute the infill_skin_overlap
+    static coord_t getSkinLineWidth(const SliceMeshStorage& mesh, const LayerIndex& layer_nr); //!< Compute the skin line width, which might be different for the first layer.
+    static coord_t getWallLineWidth0(const SliceMeshStorage& mesh, const LayerIndex& layer_nr); //!< Compute the outer wall line width, which might be different for the first layer
+    static coord_t getWallLineWidthX(const SliceMeshStorage& mesh, const LayerIndex& layer_nr); //!< Compute the inner wall line widths, which might be different for the first layer
+    static coord_t getInfillSkinOverlap(const SliceMeshStorage& mesh, const LayerIndex& layer_nr, const coord_t& innermost_wall_line_width); //!< Compute the infill_skin_overlap
 
     /*!
      * Helper function to get the walls of each part which might intersect with \p part_here
