@@ -1,28 +1,33 @@
-//Copyright (C) 2013 Ultimaker
-//Copyright (c) 2017 Ultimaker B.V.
+//Copyright (c) 2018 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
 #ifndef SUPPORT_H
 #define SUPPORT_H
 
-#include "sliceDataStorage.h"
-#include "MeshGroup.h"
-#include "commandSocket.h"
-#include "slicer.h"
+namespace cura
+{
 
-namespace cura {
+struct LayerIndex;
+class Settings;
+class SliceDataStorage;
+class SliceMeshStorage;
+class Slicer;
 
-class AreaSupport {
+class AreaSupport
+{
 public:
     /*!
-     * Move support mesh outlines from slicer data into the support storage
+     * \brief Move support mesh outlines from slicer data into the support
+     * storage.
      * 
-     * \param[out] storage Where to store the support areas
-     * \param mesh Where to get the settings from what kind of support mesh it is.
-     * \param slicer Where to get the outlines from
-     * \return Whether the mesh is used up in support and no normal mesh processing is needed
+     * \param[out] storage Where to store the support areas.
+     * \param mesh_settings Where to get the settings from what kind of support
+     * mesh it is.
+     * \param slicer Where to get the outlines from.
+     * \return Whether the mesh is used up in support and no normal mesh
+     * processing is needed.
      */
-    static bool handleSupportModifierMesh(SliceDataStorage& storage, const SettingsBaseVirtual& mesh, const Slicer* slicer);
+    static bool handleSupportModifierMesh(SliceDataStorage& storage, const Settings& mesh_settings, const Slicer* slicer);
 
     /*!
      * \brief Generate the overhang areas for all models.
@@ -132,24 +137,31 @@ private:
     static void generateOverhangAreasForMesh(SliceDataStorage& storage, SliceMeshStorage& mesh);
 
     /*!
-     * Generate support polygons over all layers for one object.
-     * 
-     * This function also handles small overhang areas (creates towers with larger diameter than just the overhang area) and single walls which could otherwise fall over.
-     * 
+     * \brief Generate support polygons over all layers for one object.
+     *
+     * This function also handles small overhang areas (creates towers with
+     * larger diameter than just the overhang area) and single walls which could
+     * otherwise fall over.
+     *
      * The anti_overhang areas are taken into account.
+     *
+     * \warning This function should be called only once for handling support
+     * meshes with drop down and once for all support meshes without drop down.
+     * The \p mesh_idx should then correspond to an empty \ref SliceMeshStorage
+     * of one support mesh with the given value of support_mesh_drop_down.
      * 
-     * \warning This function should be called only once for handling support meshes with drop down and
-     * once for all support meshes without drop down.
-     * The \p mesh_idx should then correspond to an empty \ref SliceMeshStorage of one support mesh with the given value of support_mesh_drop_down.
-     * 
-     * \param storage data storage containing the input layer outline data
-     * \param infill_settings The settings base to get the settings from which are based on the infill of the support
-     * \param roof_settings The settings base to get the settings from which are based on the top interface of the support
-     * \param bottom_settings The settings base to get the settings from which are based on the bottom interface of the support
-     * \param mesh_idx The index of the object for which to generate support areas
-     * \param layer_count total number of layers
+     * \param storage Data storage containing the input layer outline data.
+     * \param infill_settings The settings which are based on the infill of the
+     * support.
+     * \param roof_settings The settings which are based on the top interface of
+     * the support.
+     * \param bottom_settings The settings base to get the bottom interface of
+     * the support.
+     * \param mesh_idx The index of the object for which to generate support
+     * areas.
+     * \param layer_count Total number of layers.
      */
-    static void generateSupportAreasForMesh(SliceDataStorage& storage, const SettingsBaseVirtual& infill_settings, const SettingsBaseVirtual& roof_settings, const SettingsBaseVirtual& bottom_settings, const size_t mesh_idx, const size_t layer_count, std::vector<Polygons>& support_areas);
+    static void generateSupportAreasForMesh(SliceDataStorage& storage, const Settings& infill_settings, const Settings& roof_settings, const Settings& bottom_settings, const size_t mesh_idx, const size_t layer_count, std::vector<Polygons>& support_areas);
 
     /*!
      * Generate support bottom areas for a given mesh.
@@ -192,25 +204,23 @@ private:
      * going to be filled with the interface.
      * \param safety_offset An offset applied to the result to make sure
      * everything can be printed.
+     * \param outline_offset An offset applied to the result outlines.
+     * \param minimum_interface_area Minimum area size for resulting interface polygons.
      * \param[out] interface_polygons The resulting interface layer. Do not use `interface` in windows!
      */
-    static void generateSupportInterfaceLayer(Polygons& support_areas, const Polygons mesh_outlines, const coord_t safety_offset, Polygons& interface_polygons);
+    static void generateSupportInterfaceLayer(Polygons& support_areas, const Polygons mesh_outlines, const coord_t safety_offset, const coord_t outline_offset, const double minimum_interface_area, Polygons& interface_polygons);
 
     /*!
-     * Join current support layer with the support of the layer above, (make support conical) and perform smoothing etc operations.
-     * 
-     * \param supportLayer_up The support areas the layer above
-     * \param supportLayer_this The overhang areas of the current layer at hand
-     * \param supportJoinDistance The distance to be filled between two support areas
-     * \param smoothing_distance Maximal distance in the X/Y directions of a line segment which is to be smoothed out. 
-     * \param min_smoothing_area  minimal area for which to perform smoothing
-     * \param conical_support Whether the support should be conical instead of cylindrical
-     * \param conical_support_offset The offset determining the angle of the conical support
-     * \param conical_smallest_breadth The breadth of the smallest support area which is not to be redoces to a smaller size due to conical support.
-     * 
+     * \brief Join current support layer with the support of the layer above,
+     * (make support conical) and perform smoothing etc. operations.
+     * \param storage Where to store the resulting support.
+     * \param supportLayer_up The support areas the layer above.
+     * \param supportLayer_this The overhang areas of the current layer at hand.
+     * \param smoothing_distance Maximal distance in the X/Y directions of a
+     * line segment which is to be smoothed out.
      * \return The joined support areas for this layer.
      */
-    static Polygons join(const Polygons& supportLayer_up, Polygons& supportLayer_this, int64_t supportJoinDistance, int64_t smoothing_distance, int min_smoothing_area, bool conical_support, int64_t conical_support_offset, int64_t conical_smallest_breadth);
+    static Polygons join(const SliceDataStorage& storage, const Polygons& supportLayer_up, Polygons& supportLayer_this, const coord_t smoothing_distance);
 
     /*!
      * Move the support up from model (cut away polygons to ensure bottom z distance)
@@ -228,76 +238,65 @@ private:
      * \param bottom_stair_step_layer_count The max height (in nr of layers) of the support bottom stairs
      * \param support_bottom_stair_step_width The max width of the support bottom stairs
      */
-    static void moveUpFromModel(const SliceDataStorage& storage, Polygons& stair_removal, Polygons& support_areas, const int layer_idx, const int bottom_empty_layer_count, const unsigned int bottom_stair_step_layer_count, const coord_t support_bottom_stair_step_width);
+    static void moveUpFromModel(const SliceDataStorage& storage, Polygons& stair_removal, Polygons& support_areas, const size_t layer_idx, const size_t bottom_empty_layer_count, const size_t bottom_stair_step_layer_count, const coord_t support_bottom_stair_step_width);
 
     /*!
      * Joins the layer part outlines of all meshes and collects the overhang
      * points (small areas).
      * \param storage Input layer outline information.
      * \param mesh Output mesh to store the resulting overhang points in.
-     * \param minimum_diameter Diameter of the minimal area which can be
-     * supported without a specialised strut.
      */
-    static void detectOverhangPoints(
-        const SliceDataStorage& storage,
-        SliceMeshStorage& mesh,
-        const coord_t minimum_diameter
-    );
+    static void detectOverhangPoints(const SliceDataStorage& storage, SliceMeshStorage& mesh);
     
     /*!
-     * Compute the basic overhang and full overhang of a layer. 
-     * The basic overhang consists of the parts of this layer which are too far away from the layer below to be supported.
-     * The full overhang consists of the basic overhang extended toward the border of the layer below.
+     * \brief Compute the basic overhang and full overhang of a layer.
+     *
+     * The basic overhang consists of the parts of this layer which are too far
+     * away from the layer below to be supported. The full overhang consists of
+     * the basic overhang extended toward the border of the layer below.
      * 
      *             layer 2
      * layer 1 ______________|
      * _______|         ^^^^^ basic overhang
      *         ^^^^^^^^^^^^^^ full overhang
      * 
-     * \param storage The slice data storage
-     * \param mesh The mesh for which to compute the basic overhangs
-     * \param layer_idx The layer for which to compute the overhang
-     * \param max_dist_from_lower_layer The outward distance from the layer below which can be supported by it
-     * \return a pair of basic overhang and full overhang
+     * \param storage The slice data storage.
+     * \param mesh The mesh for which to compute the basic overhangs.
+     * \param layer_idx The layer for which to compute the overhang.
+     * \return A pair of basic overhang and full overhang.
      */
-    static std::pair<Polygons, Polygons> computeBasicAndFullOverhang(const SliceDataStorage& storage, const SliceMeshStorage& mesh, const unsigned int layer_idx, const int64_t max_dist_from_lower_layer);
+    static std::pair<Polygons, Polygons> computeBasicAndFullOverhang(const SliceDataStorage& storage, const SliceMeshStorage& mesh, const unsigned int layer_idx);
     
     /*!
-     * Adds tower pieces to the current support layer.
-     * From below the roof, the towers are added to the normal support layer and handled as normal support area.
-     * \param supportLayer_this The support areas in the layer for which we are creating towers/struts
-     * \param towerRoofs The parts of roofs which need to expand downward until they have the required diameter
+     * \brief Adds tower pieces to the current support layer.
+     *
+     * From below the roof, the towers are added to the normal support layer and
+     * handled as normal support area.
+     * \param settings The settings to use for towers.
+     * \param supportLayer_this The support areas in the layer for which we are
+     * creating towers/struts
+     * \param towerRoofs The parts of roofs which need to expand downward until
+     * they have the required diameter
      * \param overhang_points stores overhang_points of each layer
      * \param layer_idx The index of the layer at which to handle towers
-     * \param towerRoofExpansionDistance The offset distance which determines the angle of the tower roof tops
-     * \param supportTowerDiameter The diameter of the eventual tower, below the roof
-     * \param supportMinAreaSqrt diameter of the minimal area which can be supported without a specialized strut
      * \param layer_count total number of layers
-     * \param z_layer_distance_tower The number of layers between an overhang point and the top of a support tower
      */
     static void handleTowers(
+        const Settings& settings,
         Polygons& supportLayer_this,
         std::vector<Polygons>& towerRoofs,
         std::vector<std::vector<Polygons>>& overhang_points,
-        int layer_idx,
-        int towerRoofExpansionDistance,
-        int supportTowerDiameter,
-        int supportMinAreaSqrt,
-        int layer_count,
-        int z_layer_distance_tower
+        LayerIndex layer_idx,
+        size_t layer_count
     );
     
     /*!
-     * Adds struts (towers against a wall) to the current layer.
-     * \param supportLayer_this The areas of the layer for which to handle the wall struts.
-     * \param supportMinAreaSqrt The minimal diameter of a wall which doesn't need a strut for reinforcement
-     * \param suportTowerDiameter The diameter of the strut
+     * \brief Adds struts (towers against a wall) to the current layer.
+     * \param settings The settings to use to create the wall struts.
+     * \param supportLayer_this The areas of the layer for which to handle the
+     * wall struts.
      */
-    static void handleWallStruts(
-        Polygons& supportLayer_this,
-        int supportMinAreaSqrt,
-        int supportTowerDiameter
-    );
+    static void handleWallStruts(const Settings& settings, Polygons& supportLayer_this);
 
     /*!
      * Clean up the SupportInfillParts.
