@@ -1,8 +1,9 @@
-/** Copyright (C) 2016 Ultimaker - Released under terms of the AGPLv3 License */
-
-#include "AABB.h"
+//Copyright (c) 2018 Ultimaker B.V.
+//CuraEngine is released under the terms of the AGPLv3 or higher.
 
 #include <limits>
+#include "AABB.h"
+#include "polygon.h" //To create the AABB of a polygon.
 
 namespace cura
 {
@@ -39,9 +40,9 @@ void AABB::calculate(const Polygons& polys)
 {
     min = Point(POINT_MAX, POINT_MAX);
     max = Point(POINT_MIN, POINT_MIN);
-    for(unsigned int i=0; i<polys.size(); i++)
+    for (unsigned int i = 0; i < polys.size(); i++)
     {
-        for(unsigned int j=0; j<polys[i].size(); j++)
+        for (unsigned int j = 0; j < polys[i].size(); j++)
         {
             include(polys[i][j]);
         }
@@ -80,6 +81,12 @@ void AABB::include(Point point)
     max.Y = std::max(max.Y,point.Y);
 }
 
+void AABB::include(const AABB other)
+{
+    include(other.min);
+    include(other.max);
+}
+
 void AABB::expand(int dist)
 {
     if (min == Point(POINT_MAX, POINT_MAX) || max == Point(POINT_MIN, POINT_MIN))
@@ -92,18 +99,6 @@ void AABB::expand(int dist)
     max.Y += dist;
 }
 
-void AABB::round(const coord_t increment)
-{
-    if (increment <= 1) //If rounding to single microns we don't have to do anything. Anything lower than that doesn't make sense so ignore that.
-    {
-        return;
-    }
-    min.X = (min.X - (min.X >= 0) * (increment - 1)) / increment * increment; //Round the min vector towards negative.
-    min.Y = (min.Y - (min.Y >= 0) * (increment - 1)) / increment * increment;
-    max.X = (max.X + (max.X >= 0) * (increment - 1)) / increment * increment; //Round the max vector towards positive.
-    max.Y = (max.Y + (max.Y >= 0) * (increment - 1)) / increment * increment;
-}
-
 Polygon AABB::toPolygon() const
 {
     Polygon ret;
