@@ -1,15 +1,19 @@
+//Copyright (c) 2018 Ultimaker B.V.
+//CuraEngine is released under the terms of the AGPLv3 or higher.
+
 #ifndef SVG_H
 #define SVG_H
 
 #include <stdio.h> // for file output
 
-#include "polygon.h"
-#include "intpoint.h"
 #include "AABB.h"
-#include "logoutput.h"
+#include "IntPoint.h"
 #include "NoCopy.h"
 
-namespace cura {
+namespace cura
+{
+
+class FPoint3;
 
 class SVG : NoCopy
 {
@@ -22,7 +26,8 @@ public:
         BLUE,
         GREEN,
         YELLOW,
-        RAINBOW
+        RAINBOW,
+        NONE
     };
 
 private:
@@ -35,24 +40,35 @@ private:
     const Point border;
     const Point canvas_size;
     const double scale;
+    Color background;
 
     bool output_is_html;
 
 public:
-    SVG(const char* filename, AABB aabb, Point canvas_size = Point(1024, 1024));
+    SVG(const char* filename, AABB aabb, Point canvas_size = Point(1024, 1024), Color background = Color::NONE);
 
     ~SVG();
 
+    /*!
+     * get the scaling factor applied to convert real space to canvas space
+     */
+    double getScale() const;
+    
     /*!
      * transform a point in real space to canvas space
      */
     Point transform(const Point& p);
 
+    /*!
+     * transform a point in real space to canvas space with more precision
+     */
+    FPoint3 transformF(const Point& p);
+
     void writeComment(std::string comment);
 
-    void writeAreas(const Polygons& polygons, Color color = Color::GRAY, Color outline_color = Color::BLACK, coord_t stroke_width = 1);
+    void writeAreas(const Polygons& polygons, Color color = Color::GRAY, Color outline_color = Color::BLACK, float stroke_width = 1);
 
-    void writeAreas(ConstPolygonRef polygon, Color color = Color::GRAY, Color outline_color = Color::BLACK, coord_t stroke_width = 1);
+    void writeAreas(ConstPolygonRef polygon, Color color = Color::GRAY, Color outline_color = Color::BLACK, float stroke_width = 1);
 
     void writePoint(const Point& p, bool write_coords=false, int size = 5, Color color = Color::BLACK);
 
@@ -73,9 +89,9 @@ public:
      */
     void writeLines(std::vector<Point> polyline, Color color = Color::BLACK);
 
-    void writeLine(const Point& a, const Point& b, Color color = Color::BLACK, int stroke_width = 1);
+    void writeLine(const Point& a, const Point& b, Color color = Color::BLACK, float stroke_width = 1);
 
-    void writeLineRGB(const Point& from, const Point& to, int r = 0, int g = 0, int b = 0, int stroke_width = 1);
+    void writeLineRGB(const Point& from, const Point& to, int r = 0, int g = 0, int b = 0, float stroke_width = 1);
 
     /*!
      * \brief Draws a dashed line on the canvas from point A to point B.
@@ -93,9 +109,9 @@ public:
 
     void writeText(Point p, std::string txt, Color color = Color::BLACK, coord_t font_size = 10);
 
-    void writePolygons(const Polygons& polys, Color color = Color::BLACK, int stroke_width = 1);
+    void writePolygons(const Polygons& polys, Color color = Color::BLACK, float stroke_width = 1);
 
-    void writePolygon(ConstPolygonRef poly, Color color = Color::BLACK, int stroke_width = 1);
+    void writePolygon(ConstPolygonRef poly, Color color = Color::BLACK, float stroke_width = 1);
 
 };
 
