@@ -2,9 +2,18 @@
 #ifndef UTILS_MATH_H
 #define UTILS_MATH_H
 
+#include <cmath>
+
+
+//c++11 no longer defines M_PI, so add our own constant.
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+
 namespace cura
 {
 
+static constexpr float sqrt2 = 1.41421356237;
 
 template<typename T> inline T square(const T& a) { return a * a; }
 
